CREATE TABLE bifkv (
    key VARCHAR NOT NULL PRIMARY KEY,
    bool_val INTEGER,
    identity_id INTEGER,
    change_id INTEGER,
    change_id2 INTEGER,
    hub_id INTEGER,
    FOREIGN KEY(identity_id) REFERENCES identities(id) ON DELETE CASCADE,
    FOREIGN KEY(change_id) REFERENCES changes(id) ON DELETE CASCADE,
    FOREIGN KEY(change_id2) REFERENCES changes(id) ON DELETE CASCADE,
    FOREIGN KEY(hub_id) REFERENCES hubs(id) ON DELETE CASCADE
);
