use strict;
use warnings;
use lib 't/lib';
use Path::Tiny;
use Test::Bif;
use Test::More;

plan skip_all => 'App::bif::Build not generated'
  unless -e path(qw/lib App bif Build.pm/);

isa_ok bif(qw/ show version/), 'Bif::OK::ShowVersion';

done_testing();
