use strict;
use warnings;
use lib 't/lib';
use Test::Bif;
use Test::Fatal;
use Test::More;

run_in_tempdir {

    isa_ok exception { bif(qw/wlog/) }, 'Bif::Error::UserRepoNotFound';
    bif(qw/init/);

    isa_ok bif(qw/wlog/),    'Bif::OK::WorkLogShort';
    isa_ok bif(qw/wlog -l/), 'Bif::OK::WorkLogLong';
};

done_testing();
